@@ -2,22 +2,27 @@
 
 # Internal Libraries
 import os
+from typing import List, Tuple
+
+import numpy as np
 
-import auto_editor
 from auto_editor.utils.effects import Effect
+from auto_editor.utils.log import Log
 from auto_editor.utils.func import fnone, set_output_name, append_filename
 
-def get_chunks(inp, speeds, segment, fps, args, log, audio_samples=None, sample_rate=None):
-    from auto_editor.cutting import (combine_audio_motion, combine_segment,
-        apply_spacing_rules, apply_mark_as, apply_frame_margin, seconds_to_frames, cook)
+def get_chunks(inp, segment, fps, args, log, audio_samples=None, sample_rate=None):
+    from auto_editor.cutting import (combine_audio_motion, combine_segment, to_speed_list,
+        set_range, chunkify, apply_mark_as, apply_margin, seconds_to_frames, cook)
 
-    frame_margin = seconds_to_frames(args.frame_margin, fps)
+    start_margin, end_margin = args.frame_margin
+
+    start_margin = seconds_to_frames(start_margin, fps)
+    end_margin = seconds_to_frames(end_margin, fps)
     min_clip = seconds_to_frames(args.min_clip_length, fps)
     min_cut = seconds_to_frames(args.min_cut_length, fps)
 
     def get_has_loud(inp, args, fps, audio_samples, sample_rate, log):
-        import numpy as np
-
+        # type: (...) -> np.ndarray
         from auto_editor.analyze.generic import get_np_list
 
         if(args.edit_based_on == 'none'):
@@ -43,7 +48,7 @@ def get_chunks(inp, speeds, segment, fps, args, log, audio_samples=None, sample_
                 motion_list = get_np_list(inp, audio_samples, sample_rate, fps, np.ones)
             else:
                 motion_list = motion_detection(inp, args.motion_threshold, log,
-                    width=args.width, dilates=args.dilates, blur=args.blur)
+                    width=args.md_width, dilates=args.md_dilates, blur=args.md_blur)
 
         if(audio_list is not None and motion_list is not None):
             if(len(audio_list) > len(motion_list)):
@@ -57,31 +62,44 @@ def get_chunks(inp, speeds, segment, fps, args, log, audio_samples=None, sample_
     has_loud_length = len(has_loud)
     has_loud = apply_mark_as(has_loud, has_loud_length, fps, args, log)
     has_loud = cook(has_loud, min_clip, min_cut)
-    has_loud = apply_frame_margin(has_loud, has_loud_length, frame_margin)
+    has_loud = apply_margin(has_loud, has_loud_length, start_margin, end_margin)
 
     if(segment is not None):
         has_loud = combine_segment(has_loud, segment, fps)
+
     # Remove small clips/cuts created by applying other rules.
     has_loud = cook(has_loud, min_clip, min_cut)
-    return apply_spacing_rules(has_loud, has_loud_length, min_clip, min_cut, speeds,
-        fps, args, log)
+
+    speed_list = to_speed_list(has_loud, args.video_speed, args.silent_speed)
+
+    if(args.cut_out != []):
+        speed_list = set_range(speed_list, args.cut_out, fps, 99999, log)
+
+    if(args.add_in != []):
+        speed_list = set_range(speed_list, args.add_in, fps, args.video_speed, log)
+
+    if(args.set_speed_for_range != []):
+        for item in args.set_speed_for_range:
+            speed_list = set_range(speed_list, [item[1:]], fps, item[0], log)
+
+    return chunkify(speed_list, has_loud_length)
 
 
-def edit_media(i, inp, ffmpeg, args, progress, speeds, segment, exporting_to_editor,
-    data_file, temp, log):
+def edit_media(i, inp, ffmpeg, args, progress, segment, exporting_to_editor, data_file,
+    temp, log):
     from auto_editor.utils.container import get_rules
 
     chunks = None
     if(inp.ext == '.json'):
         from auto_editor.formats.make_json import read_json_cutlist
 
-        input_path, chunks, speeds = read_json_cutlist(inp.path, auto_editor.version, log)
+        input_path, chunks = read_json_cutlist(inp.path, log)
         inp = ffmpeg.file_info(input_path)
 
         output_path = set_output_name(inp.path, inp.ext, data_file, args)
     else:
         output_path = args.output_file[i]
-        if(not os.path.isdir(inp.path) and '.' not in output_path):
+        if(not os.path.isdir(inp.path) and os.path.splitext(output_path)[1] == ''):
             output_path = set_output_name(output_path, inp.ext, data_file, args)
 
     log.debug('{} -> {}'.format(inp.path, output_path))
@@ -110,19 +128,25 @@ def edit_media(i, inp, ffmpeg, args, progress, speeds, segment, exporting_to_edi
         if(vcodec in rules['disallow_v']):
             log.error(codec_error.format(vcodec, output_container))
 
-    if(not fnone(args.audio_codec)):
-        if(rules['astrict'] and args.audio_codec not in rules['acodecs']):
-            log.error(codec_error.format(args.audio_codec, output_container))
+    acodec = args.audio_codec
+    if(acodec == 'copy'):
+        acodec = inp.audio_streams[0]['codec']
+        log.debug(f'Settings acodec to {acodec}')
 
-        if(args.audio_codec in rules['disallow_a']):
-            log.error(codec_error.format(args.audio_codec, output_container))
+    if(acodec not in ['unset', 'auto']):
+        if(rules['astrict'] and acodec not in rules['acodecs']):
+            log.error(codec_error.format(acodec, output_container))
+
+        if(acodec in rules['disallow_a']):
+            log.error(codec_error.format(acodec, output_container))
 
     if(args.keep_tracks_seperate and rules['max_audio_streams'] == 1):
         log.warning(
             "'{}' container doesn't support multiple audio tracks.".format(container)
         )
 
-    if(os.path.isfile(output_path) and inp.path != output_path):
+    if(os.path.isfile(output_path) and inp.path != output_path and not args.preview
+        and not args.timeline):
         log.debug('Removing already existing file: {}'.format(output_path))
         os.remove(output_path)
 
@@ -130,14 +154,12 @@ def edit_media(i, inp, ffmpeg, args, progress, speeds, segment, exporting_to_edi
     tracks = len(inp.audio_streams)
     audio_file = len(inp.video_streams) == 0 and tracks == 1
 
-    if(args.force_fps_to is not None):
-        fps = args.force_fps_to
-    elif(inp.fps is None):
+    if(inp.fps is None):
         fps = 30
+    elif(exporting_to_editor):
+        fps = int(float(inp.fps))
     else:
         fps = float(inp.fps)
-        if(exporting_to_editor):
-            fps = int(fps)
 
     if(fps < 1):
         log.error('{}: Frame rate cannot be below 1. fps: {}'.format(inp.basename, fps))
@@ -183,7 +205,7 @@ def edit_media(i, inp, ffmpeg, args, progress, speeds, segment, exporting_to_edi
         cmd.extend(['-map', '0:a:{}'.format(t), '-ac', '2',
             os.path.join(temp, '{}.wav'.format(t))])
     cmd.extend(['-map', '0:v:0'])
-    if(args.has_vfr == 'unset' and len(inp.video_streams) > 0):
+    if(args.has_vfr == 'unset' and len(inp.video_streams) > 0 and not args.preview):
         log.conwrite('Extracting audio / detecting VFR')
         cmd.extend(['-vf', 'vfrdet', '-f', 'null', '-'])
         has_vfr = has_VFR(cmd, log)
@@ -213,44 +235,43 @@ def edit_media(i, inp, ffmpeg, args, progress, speeds, segment, exporting_to_edi
         from auto_editor.scipy.wavfile import read
         sample_rate, audio_samples = read(temp_file)
 
-    effects = Effect(args, log, _vars={
-        'silent_threshold': args.silent_threshold
-        })
-    effects.audio_samples = audio_samples
-    effects.sample_rate = sample_rate
-
     log.debug('Frame Rate: {}'.format(fps))
     if(chunks is None):
-        chunks = get_chunks(inp, speeds, segment, fps, args, log, audio_samples,
-            sample_rate)
+        chunks = get_chunks(inp, segment, fps, args, log, audio_samples, sample_rate)
 
-    def is_clip(chunk):
-        # type: (list) -> bool
-        return speeds[chunk[2]] != 99999
+    def is_clip(chunk: Tuple[int, int, float]) -> bool:
+        return chunk[2] != 99999
 
-    def number_of_cuts(chunks, speeds):
-        # type: (list, list) -> int
+    def number_of_cuts(chunks):
+        # type: (List[Tuple[int, int, float]]) -> int
         return len(list(filter(is_clip, chunks)))
 
-    def get_clips(chunks, speeds):
+    def get_clips(chunks):
+        # type: (List[Tuple[int, int, float]]) -> List[List[int, int, float]]
         clips = []
         for chunk in chunks:
             if(is_clip(chunk)):
-                clips.append([chunk[0], chunk[1], speeds[chunk[2]] * 100])
+                clips.append([chunk[0], chunk[1], chunk[2] * 100])
         return clips
 
-    num_cuts = number_of_cuts(chunks, speeds)
-    clips = get_clips(chunks, speeds)
+    num_cuts = number_of_cuts(chunks)
+    clips = get_clips(chunks)
+
+    effects = Effect(args, log, _vars={})
 
     if(args.export_as_json):
         from auto_editor.formats.make_json import make_json_cutlist
-        make_json_cutlist(inp.path, output_path, auto_editor.version, chunks, speeds,
-            log)
+        make_json_cutlist(inp.path, output_path, chunks, log)
         return num_cuts, output_path
 
+    if(args.timeline):
+        from auto_editor.formats.make_json import make_json_cutlist
+        make_json_cutlist(inp.path, 0, chunks, log)
+        return num_cuts, None
+
     if(args.preview):
         from auto_editor.preview import preview
-        preview(inp, chunks, speeds, log)
+        preview(inp, chunks, log)
         return num_cuts, None
 
     if(args.export_to_premiere):
@@ -259,7 +280,7 @@ def edit_media(i, inp, ffmpeg, args, progress, speeds, segment, exporting_to_edi
             fps, log)
         return num_cuts, output_path
 
-    if(args.export_to_final_cut_pro or args.export_to_resolve):
+    if(args.export_to_final_cut_pro):
         from auto_editor.formats.final_cut_pro import fcp_xml
 
         total_frames = chunks[len(chunks) - 1][1]
@@ -290,11 +311,9 @@ def edit_media(i, inp, ffmpeg, args, progress, speeds, segment, exporting_to_edi
     def make_media(inp, chunks, output_path):
         from auto_editor.utils.video import mux_quality_media
 
-        spedup = None
-
         if(rules['allow_subtitle']):
             from auto_editor.render.subtitle import cut_subtitles
-            cut_subtitles(ffmpeg, inp, chunks, speeds, fps, temp, log)
+            cut_subtitles(ffmpeg, inp, chunks, fps, temp, log)
 
         if(rules['allow_audio']):
             from auto_editor.render.audio import make_new_audio
@@ -302,37 +321,38 @@ def edit_media(i, inp, ffmpeg, args, progress, speeds, segment, exporting_to_edi
             for t in range(tracks):
                 temp_file = os.path.join(temp, '{}.wav'.format(t))
                 new_file = os.path.join(temp, 'new{}.wav'.format(t))
-                make_new_audio(temp_file, new_file, chunks, speeds, log, fps, progress)
+                make_new_audio(temp_file, new_file, chunks, log, fps, progress)
 
                 if(not os.path.isfile(new_file)):
                     log.bug('Audio file not created.')
 
-        apply_video = True
+        video_stuff = []
 
-        if(rules['allow_video'] and len(inp.video_streams) > 0):
-            from auto_editor.render.av import render_av
-            spedup, apply_video = render_av(ffmpeg, inp, args, chunks, speeds, fps,
-                has_vfr, progress, effects, rules, temp, log)
+        if(rules['allow_video']):
+            from auto_editor.render.video import render_av
+            for v, vid in enumerate(inp.video_streams):
+                if(vid['codec'] in ['png', 'jpeg']):
+                    video_stuff.append(('image', None, None))
+                else:
+                    video_stuff.append(render_av(ffmpeg, v, inp, args, chunks, fps,
+                        has_vfr, progress, effects, rules, temp, log))
 
-            if(log.is_debug):
-                log.debug('Writing the output file.')
-            else:
-                log.conwrite('Writing the output file.')
+        log.conwrite('Writing the output file.')
 
-        mux_quality_media(ffmpeg, spedup, rules, output_path, output_container,
-            apply_video, args, inp, temp, log)
+        mux_quality_media(ffmpeg, video_stuff, rules, output_path, output_container,
+            args, inp, temp, log)
         if(output_path is not None and not os.path.isfile(output_path)):
-            log.bug('The file {} was not created.'.format(output_path))
-
-
-    total_frames = chunks[len(chunks) - 1][1]
+            log.bug(f'The file {output_path} was not created.')
 
     if(args.export_as_clip_sequence):
-        speeds.append(99999)
-        for i, chunk in enumerate(chunks):
-            if(speeds[chunk[2]] == 99999):
+        total_frames = chunks[-1][1]
+        clip_num = 0
+        for chunk in chunks:
+            if(chunk[2] == 99999):
                 continue
-            make_media(inp, pad_chunk(chunk, total_frames), append_filename(output_path, '-{}'.format(i)))
+            make_media(inp, pad_chunk(chunk, total_frames),
+                append_filename(output_path, f'-{clip_num}'))
+            clip_num += 1
     else:
         make_media(inp, chunks, output_path)
     return num_cuts, output_path
