@@ -18,84 +18,6 @@ def regex_match(regex, text):
     return None
 
 
-class File:
-    def __repr__(self):
-        return str(self.__dict__)
-
-    def __init__(self, ffmpeg, path):
-        self.path = path
-        self.abspath = os.path.abspath(path)
-        self.basename = os.path.basename(path)
-        self.dirname = os.path.dirname(os.path.abspath(path))
-
-        f_name, f_ext = os.path.splitext(path)
-        self.name = f_name
-        self.ext = f_ext
-
-        info = get_stdout([ffmpeg.path, '-hide_banner', '-i', path])
-
-        self.duration = regex_match(r'Duration:\s(?P<match>[\d:.]+),', info)
-        self.bitrate = regex_match(r'bitrate:\s(?P<match>\d+\skb\/s)', info)
-
-        self.metadata = {}
-        active = False
-        active_key = None
-
-        for line in info.split('\n'):
-            if(active):
-                if(re.search(r'^\s*[A-Z][a-z_]*', line)):
-                    break
-
-                key = regex_match(r'^\s*(?P<match>[a-z_]+)', line)
-                body = regex_match(r'^\s*[a-z_]*\s*:\s(?P<match>[\w\W]*)', line)
-
-                if(key is None):
-                    self.metadata[active_key] += '\n' + body
-                else:
-                    self.metadata[key] = body
-                    active_key = key
-
-            if(re.search(r'^\s\sMetadata:', line)):
-                active = True
-
-        video_streams = []
-        audio_streams = []
-        subtitle_streams = []
-        fps = None
-
-        sub_exts = {'mov_text': 'srt', 'ass': 'ass', 'webvtt': 'vtt'}
-
-        for line in info.split('\n'):
-            if(re.search(r'Stream #', line)):
-                s_data = {}
-                if(re.search(r'Video:', line)):
-                    s_data['width'] = regex_match(r'(?P<match>\d+)x\d+[\s,]', line)
-                    s_data['height'] = regex_match(r'\d+x(?P<match>\d+)[\s,]', line)
-                    s_data['codec'] = regex_match(r'Video:\s(?P<match>\w+)', line)
-                    s_data['bitrate'] = regex_match(r'\s(?P<match>\d+\skb\/s)', line)
-                    if(fps is None):
-                        fps = regex_match(r'\s(?P<match>[\d\.]+)\stbr', line)
-                    video_streams.append(s_data)
-
-                elif(re.search(r'Audio:', line)):
-                    s_data['codec'] = regex_match(r'Audio:\s(?P<match>\w+)', line)
-                    s_data['samplerate'] = regex_match(r'(?P<match>\d+)\sHz', line)
-                    s_data['bitrate'] = regex_match(r'\s(?P<match>\d+\skb\/s)', line)
-                    s_data['lang'] = regex_match(r'Stream #\d+:\d+\((?P<match>\w+)\)', line)
-                    audio_streams.append(s_data)
-
-                elif(re.search(r'Subtitle:', line)):
-                    s_data['lang'] = regex_match(r'Stream #\d+:\d+\((?P<match>\w+)\)', line)
-                    s_data['codec'] = regex_match(r'Subtitle:\s(?P<match>\w+)', line)
-                    s_data['ext'] = sub_exts.get(s_data['codec'], 'vtt')
-                    subtitle_streams.append(s_data)
-
-        self.fps = fps
-        self.video_streams = video_streams
-        self.audio_streams = audio_streams
-        self.subtitle_streams = subtitle_streams
-
-
 class FFmpeg():
 
     @staticmethod
@@ -126,15 +48,17 @@ class FFmpeg():
 
             Log().error('ffmpeg must be installed and on PATH.')
 
-    def print(self, message):
+    def print(self, message: str):
         if(self.debug):
             print('FFmpeg: {}'.format(message), file=sys.stderr)
 
     def print_cmd(self, cmd):
+        # type: (list[str]) -> None
         if(self.debug):
             print('FFmpeg run: {}\n'.format(' '.join(cmd)), file=sys.stderr)
 
     def run(self, cmd):
+        # type: (list[str]) -> None
         cmd = [self.path, '-y', '-hide_banner'] + cmd
         if(not self.debug):
             cmd.extend(['-nostats', '-loglevel', 'error'])
@@ -149,7 +73,6 @@ class FFmpeg():
         process.stdin.close()
         output = stderr.decode()
 
-        import re
         error_list = [
             r"Unknown encoder '.*'",
             r"-q:v qscale not available for encoder\. Use -b:v bitrate instead\.",
@@ -161,7 +84,6 @@ class FFmpeg():
 
         if(self.debug):
             print('stderr: {}'.format(stderr.decode()))
-            print('stdout: {}'.format(stdout.decode()))
 
         for item in error_list:
             check = re.search(item, output)
@@ -170,7 +92,6 @@ class FFmpeg():
 
         if(show_out and not self.debug):
             print('stderr: {}'.format(stderr.decode()))
-            print('stdout: {}'.format(stdout.decode()))
 
     def file_info(self, path):
         return File(self, path)
@@ -188,3 +109,83 @@ class FFmpeg():
         output = get_stdout(cmd)
         self.print(output)
         return output
+
+
+class File:
+    __slots__ = ('path', 'abspath', 'basename', 'dirname', 'name', 'ext', 'duration',
+        'bitrate', 'metadata', 'fps', 'video_streams', 'audio_streams', 'subtitle_streams')
+
+    def __init__(self, ffmpeg: FFmpeg, path: str):
+        self.path = path
+        self.abspath = os.path.abspath(path)
+        self.basename = os.path.basename(path)
+        self.dirname = os.path.dirname(os.path.abspath(path))
+        self.name, self.ext = os.path.splitext(path)
+
+        info = get_stdout([ffmpeg.path, '-hide_banner', '-i', path])
+
+        self.duration = regex_match(r'Duration:\s(?P<match>[\d:.]+),', info)
+        self.bitrate = regex_match(r'bitrate:\s(?P<match>\d+\skb\/s)', info)
+
+        self.metadata = {}
+        active = False
+        active_key = None
+
+        for line in info.split('\n'):
+            if(active):
+                if(re.search(r'^\s*[A-Z][a-z_]*', line)):
+                    break
+
+                key = regex_match(r'^\s*(?P<match>[a-z_]+)', line)
+                body = regex_match(r'^\s*[a-z_]*\s*:\s(?P<match>[\w\W]*)', line)
+
+                if(body is not None):
+                    if(key is None):
+                        if(active_key is not None):
+                            self.metadata[active_key] += '\n' + body
+                    else:
+                        self.metadata[key] = body
+                        active_key = key
+
+            if(re.search(r'^\s\sMetadata:', line)):
+                active = True
+
+        video_streams = []
+        audio_streams = []
+        subtitle_streams = []
+        fps = None
+
+        sub_exts = {'mov_text': 'srt', 'ass': 'ass', 'webvtt': 'vtt'}
+
+        for line in info.split('\n'):
+            if(re.search(r'Stream #', line)):
+                s_data = {}
+                if(re.search(r'Video:', line)):
+                    s_data['width'] = regex_match(r'(?P<match>\d+)x\d+[\s,]', line)
+                    s_data['height'] = regex_match(r'\d+x(?P<match>\d+)[\s,]', line)
+                    s_data['codec'] = regex_match(r'Video:\s(?P<match>\w+)', line)
+                    s_data['bitrate'] = regex_match(r'\s(?P<match>\d+\skb\/s)', line)
+                    s_data['fps'] = regex_match(r'\s(?P<match>[\d\.]+)\stbr', line)
+                    s_data['lang'] = regex_match(r'Stream #\d+:\d+\((?P<match>\w+)\)', line)
+
+                    if(fps is None):
+                        fps = s_data['fps']
+                    video_streams.append(s_data)
+
+                elif(re.search(r'Audio:', line)):
+                    s_data['codec'] = regex_match(r'Audio:\s(?P<match>\w+)', line)
+                    s_data['samplerate'] = regex_match(r'(?P<match>\d+)\sHz', line)
+                    s_data['bitrate'] = regex_match(r'\s(?P<match>\d+\skb\/s)', line)
+                    s_data['lang'] = regex_match(r'Stream #\d+:\d+\((?P<match>\w+)\)', line)
+                    audio_streams.append(s_data)
+
+                elif(re.search(r'Subtitle:', line)):
+                    s_data['lang'] = regex_match(r'Stream #\d+:\d+\((?P<match>\w+)\)', line)
+                    s_data['codec'] = regex_match(r'Subtitle:\s(?P<match>\w+)', line)
+                    s_data['ext'] = sub_exts.get(s_data['codec'], 'vtt')
+                    subtitle_streams.append(s_data)
+
+        self.fps = fps
+        self.video_streams = video_streams
+        self.audio_streams = audio_streams
+        self.subtitle_streams = subtitle_streams
