@@ -14,12 +14,11 @@ from time import perf_counter
 
 # Included Libraries
 from auto_editor.utils.func import clean_list
-from auto_editor.utils.log import Log
 import auto_editor.vanparse as vanparse
 
 def test_options(parser):
     parser.add_argument('--ffprobe_location', default='ffprobe',
-        help='point to your custom ffmpeg file.')
+        help='point to your custom ffprobe file.')
     parser.add_argument('--only', '-n', nargs='*')
     parser.add_argument('--help', '-h', action='store_true',
         help='print info about the program or an option and exit.')
@@ -67,6 +66,14 @@ class FFprobe():
     def getSampleRate(self, file, track=0):
         return self._get(file, 'sample_rate', 'a', track)
 
+    def getVLanguage(self, file):
+        return self.pipe(['-show_entries', 'stream=index:stream_tags=language',
+            '-select_streams', 'v', '-of', 'compact=p=0:nk=1', file]).strip()
+
+    def getALanguage(self, file):
+        return self.pipe(['-show_entries', 'stream=index:stream_tags=language',
+            '-select_streams', 'a', '-of', 'compact=p=0:nk=1', file]).strip()
+
     def AudioBitRate(self, file):
 
         def bitrate_format(num):
@@ -231,7 +238,12 @@ def main(sys_args=None):
     if(sys_args is None):
         sys_args = sys.args[1:]
 
-    args = parser.parse_args(sys_args, Log(), 'test')
+    try:
+        args = parser.parse_args(sys_args)
+    except vanparse.ParserError as e:
+        print(e)
+        sys.exit(1)
+
     ffprobe = FFprobe(args.ffprobe_location)
 
     tester = Tester(args)
@@ -241,10 +253,6 @@ def main(sys_args=None):
         run_program(['-h'])
         run_program(['--frame_margin', '--help'])
         run_program(['--frame_margin', '-h'])
-        run_program(['exportMediaOps', '--help'])
-        run_program(['exportMediaOps', '-h'])
-        run_program(['progressOps', '-h'])
-
         run_program(['--help', '--help'])
         run_program(['-h', '--help'])
         run_program(['--help', '-h'])
@@ -265,6 +273,12 @@ def main(sys_args=None):
             sys.exit(1)
     tester.run_test('version_tests', version_debug)
 
+
+    def parser_test():
+        check_for_error(['example.mp4', '--block'], 'needs argument')
+
+    tester.run_test('parser_test', parser_test, allow_fail=True)
+
     def subtitle_tests():
         from auto_editor.render.subtitle import SubtitleParser
         test = SubtitleParser()
@@ -276,10 +290,9 @@ def main(sys_args=None):
             [40, 50, "E"],
             [50, 60, "F"],
         ]
-        speeds = [99999, 1]
-        chunks = [[0, 10, 1], [10, 20, 0], [20, 30, 1], [30, 40, 0], [40, 50, 1],
-            [50, 60, 0]]
-        test.edit(chunks, speeds)
+        chunks = [(0, 10, 1), (10, 20, 99999), (20, 30, 1), (30, 40, 99999), (40, 50, 1),
+            (50, 60, 99999)]
+        test.edit(chunks)
 
         if(test.contents != [[0, 10, "A"], [10, 20, "C"], [20, 30, "E"]]):
             raise ValueError('Incorrect subtitle results.')
@@ -324,6 +337,8 @@ def main(sys_args=None):
             [ffprobe.getResolution, '1280x720'],
             [ffprobe.getVideoCodec, 'h264'],
             [ffprobe.getSampleRate, '48000'],
+            [ffprobe.getVLanguage, '0|eng'],
+            [ffprobe.getALanguage, '1|eng'],
         )
         run_program(['example.mp4', '--video_codec', 'uncompressed', '--has_vfr', 'no'])
         inspect(
@@ -397,8 +412,8 @@ def main(sys_args=None):
         run_program(['example.mp4', '-m', '3'])
         run_program(['example.mp4', '--margin', '3'])
         run_program(['example.mp4', '-m', '0.3sec'])
-        run_program(['example.mp4', '-m', '6f'])
-        run_program(['example.mp4', '-m', '5 frames'])
+        run_program(['example.mp4', '-m', '6f,-3secs'])
+        run_program(['example.mp4', '-m', '3,5 frames'])
         run_program(['example.mp4', '-m', '0.4 seconds'])
     tester.run_test('margin_tests', margin_tests)
 
@@ -429,8 +444,8 @@ def main(sys_args=None):
     tester.run_test('output_extension', output_extension)
 
     def progress_ops_test():
-        run_program(['example.mp4', '--has_vfr', 'no', 'progressOps', '--machine_readable_progress'])
-        run_program(['example.mp4', '--has_vfr', 'no', 'progressOps', '--no_progress'])
+        run_program(['example.mp4', '--has_vfr', 'no', '--machine_readable_progress'])
+        run_program(['example.mp4', '--has_vfr', 'no', '--no_progress'])
     tester.run_test('progress_ops_test', progress_ops_test)
 
     def silent_threshold():
@@ -467,9 +482,8 @@ def main(sys_args=None):
     tester.run_test('scale_tests', scale_tests)
 
     def various_errors_test():
-        check_for_error(['example.mp4', '--zoom', '0', '--cut_out', '60,end'])
-        check_for_error(['example.mp4', '--zoom', '0,60', '--cut_out', '60,end'])
-        check_for_error(['example.mp4', '--rectangle', '0,60', '--cut_out', '60,end'])
+        check_for_error(['example.mp4', '--add_rectangle', '0,60', '--cut_out', '60,end'])
+
     tester.run_test('various_errors_test', various_errors_test)
 
     def effect_tests():
@@ -480,27 +494,26 @@ def main(sys_args=None):
             [ffprobe.getFrameRate, 30.0],
             [ffprobe.getResolution, '640x360'],
         )
-
-        run_program(['testsrc.mp4', '--mark_as_loud', 'start,end', '--zoom', '10,60,2'])
-
-        run_program(['example.mp4', '--mark_as_loud', 'start,end', '--rectangle',
-            'audio>0.05,audio<0.05,20,50,50,100', 'audio>0.1,audio<0.1,120,50,150,100'])
-
-        run_program(['testsrc.mp4', '--mark_as_loud', 'start,end', '--zoom',
-            'start,end,1,0.5,centerX,centerY,linear', '--scale', '0.5'])
-        inspect(
-            'testsrc_ALTERED.mp4',
-            [ffprobe.getFrameRate, 30.0],
-            [ffprobe.getResolution, '320x180'],
-        )
-        run_program(['testsrc.mp4', '--mark_as_loud', 'start,end', '--rectangle',
+        run_program(['testsrc.mp4', '--mark_as_loud', 'start,end', '--add_rectangle',
             '0,30,0,200,100,300,#43FA56,10'])
         os.remove('testsrc_ALTERED.mp4')
         os.remove('testsrc.mp4')
     tester.run_test('effect_tests', effect_tests,
-        description='test the zoom and rectangle options',
+        description='test the rectangle object',
         cleanup=clean_all)
 
+    def render_text():
+        run_program(['example.mp4', '--has-vfr', 'no', '--add-text', 'start,end,This is my text'])
+
+    tester.run_test('render_text', render_text)
+
+    def check_font_error():
+        check_for_error(
+            ['example.mp4', '--has-vfr', 'no', '--add-text', 'start,end,text,0,0,30,notafont'],
+            'not found')
+
+    tester.run_test('check_font_error', check_font_error)
+
     def export_tests():
         for item in os.listdir('resources'):
             if('man_on_green_screen' in item or item.startswith('.') or '_ALTERED' in item
@@ -509,7 +522,6 @@ def main(sys_args=None):
             item = 'resources/{}'.format(item)
             run_program([item])
             run_program([item, '-exp'])
-            run_program([item, '-exr'])
             run_program([item, '-exf'])
             run_program([item, '-exs'])
             run_program([item, '--export_as_clip_sequence'])
