@@ -1,9 +1,11 @@
 '''vanparse.py'''
 
-import os
 import sys
 import difflib
 
+class ParserError(Exception):
+    pass
+
 def indent(text, prefix, predicate=None):
     if predicate is None:
         def predicate(line):
@@ -35,21 +37,20 @@ def out(text):
     print('\n'.join(wrapped_lines))
 
 def print_option_help(args, option):
-    text = ''
-    if(option['action'] == 'grouping'):
-        text += '  {}:\n'.format(option['names'][0])
-    else:
-        text += '  ' + ', '.join(option['names']) + '\n    ' + option['help'] + '\n\n'
-        if(option['keywords'] != []):
-            text += '    Arguments:\n    '
-            for n, item in enumerate(option['keywords']):
-                [[k, v]] = item.items()
+    text = '  ' + ', '.join(option['names']) + '\n    ' + option['help'] + '\n\n'
+    if(option['keywords'] != []):
+        text += '    Arguments:\n    '
+        for n, item in enumerate(option['keywords']):
+            [[k, v]] = item.items()
+
+            if(k != 'type'):
+                _, default = v
                 text += '{' + k
-                text += '' if v == '' else '=' + str(v)
+                text += '' if default == '' else '=' + str(default)
                 text += '}' if n + 1 == len(option['keywords']) else '},'
 
-        if(option['extra'] != ''):
-            text += '{}\n\n'.format(indent(option['extra'], '    '))
+    if(option['manual'] != ''):
+        text += '{}\n\n'.format(indent(option['manual'], '    '))
 
     if(option['keywords'] != []):
         pass
@@ -61,43 +62,33 @@ def print_option_help(args, option):
 
         if(option['choices'] is not None):
             text += '    choices: ' +  ', '.join(option['choices']) + '\n'
-    elif(option['action'] == 'grouping'):
-        for options in args:
-            for op in options:
-                if(op['group'] == option['names'][0]):
-                    text += '  ' + ', '.join(op['names']) + ': ' + op['help'] + '\n'
     elif(option['action'] in ['store_true', 'store_false']):
         text += '    type: flag\n'
     else:
         text += '    type: unknown\n'
 
-    if(option['group'] is not None):
-        text += '    group: ' + option['group'] + '\n'
     out(text)
 
-def print_program_help(root, the_args):
+def print_program_help(the_args):
     text = ''
     for options in the_args:
         for option in options:
-            if(not option['hidden']):
-                if(option['action'] == 'grouping'):
-                    text += "\n  {}:\n".format(option['names'][0])
-                else:
-                    text += '  ' + ', '.join(option['names']) + ': ' + option['help'] + '\n'
+            if(option['action'] == 'text'):
+                text += '\n  ' + option['names'][0] + '\n'
+            elif(option['action'] == 'blank'):
+                text += '\n'
+            elif(not option['hidden']):
+                text += '  ' + ', '.join(option['names']) + ': ' + option['help'] + '\n'
     text += '\n'
-    if(root == 'auto-editor'):
-        text += ('  Have an issue? Make an issue. Visit '
-            'https://github.com/wyattblue/auto-editor/issues\n\n  The help option '
-            'can also be used on a specific option:\n     auto-editor '
-            '--frame_margin --help\n')
     out(text)
 
 def get_option(item, the_args):
     for options in the_args:
         for option in options:
-            dash = list(map(lambda n: n.replace('_', '-'), option['names']))
-            if((item in option['names'] or item in dash)):
-                return option
+            if(option['action'] != 'text' and option['action'] != 'blank'):
+                dash = list(map(lambda n: n.replace('_', '-'), option['names']))
+                if((item in option['names'] or item in dash)):
+                    return option
     return None
 
 def _to_key(val):
@@ -122,7 +113,7 @@ class ArgumentParser():
             'help': '',
             'keywords': [],
             'hidden': False,
-            'extra': '',
+            'manual': '',
         }
 
     def add_argument(self, *args, **kwargs):
@@ -140,7 +131,16 @@ class ArgumentParser():
 
         self.args.append(my_dict)
 
-    def parse_args(self, sys_args, log, root):
+    def add_text(self, text):
+        self.args.append({
+            'names': [text],
+            'action': 'text',
+        });
+
+    def add_blank(self):
+        self.args.append({'action': 'blank'});
+
+    def parse_args(self, sys_args):
         if(sys_args == [] and self.description):
             out(self.description)
             sys.exit()
@@ -149,7 +149,7 @@ class ArgumentParser():
             out('{} version {}'.format(self.program_name, self._version))
             sys.exit()
 
-        return ParseOptions(sys_args, log, root, self.args)
+        return ParseOptions(sys_args, self.args)
 
 
 """
@@ -163,9 +163,12 @@ class ArgumentParser():
 
 class ParseOptions():
 
-    def parse_parameters(self, val, op):
+    @staticmethod
+    def parse_parameters(unsplit_arguments, op):
 
-        # TODO: allow out-of-order arguments with keyword syntax.
+        ARG_SEP = ','
+        KEYWORD_SEP = '='
+        REQUIRED = ''
 
         dic = {}
         keys = []
@@ -174,58 +177,40 @@ class ParseOptions():
             keys.append(k)
             dic[k] = v
 
-        for i, item in enumerate(val.split(',')):
+        allow_positional_args = True
+
+        for i, item in enumerate(unsplit_arguments.split(ARG_SEP)):
             if(i+1 > len(keys)):
-                print(f"Error! Too many arguments, starting with '{item}'", file=sys.stderr)
-                sys.exit(1)
+                raise ParserError(f"Too many arguments, starting with '{item}'.")
 
-            dic[keys[i]] = item
+            if(KEYWORD_SEP in item):
+                allow_positional_args = False
 
-        # Check if any positional args are not specified.
-        for key, item in dic.items():
-            if(item == ''):
-                print(f"Error! parameter '{key}' is required.", file=sys.stderr)
-                sys.exit(1)
+                parameters = item.split(KEYWORD_SEP)
+                if(len(parameters) > 2):
+                    raise ParserError(f"Invalid Syntax: '{item}'.")
+                key, val = parameters
+                if(key not in dic):
+                    raise ParserError(f"Parameter '{key}' does not exist.")
+                dic[key] = (dic[key][0], val)
+            elif(allow_positional_args):
+                dic[keys[i]] = (dic[keys[i]][0], item)
+            else:
+                raise ParserError('Positional argument follows keyword argument.')
+
+        # Check if any required args are not specified.
+        for key, type_val in dic.items():
+            if(type_val[1] == REQUIRED):
+                raise ParserError(f"Parameter '{key}' is required.")
         return dic
 
-    def set_config(self, config_path, root):
-        if(not os.path.isfile(config_path)):
-            return
-
-        with open(config_path, 'r') as file:
-            lines = file.readlines()
-
-        # Set attributes based on the config file to act as the new defaults.
-        for item in lines:
-            if('#' in item):
-                item = item[: item.index('#')]
-            item = item.replace(' ', '')
-            if(item.strip() == '' or (not item.startswith(root))):
-                continue
-            value = item[item.index('=')+1 :]
-
-            if(value[0] == "'" and value[-1] == "'"):
-                value = value[1:-1]
-            elif(value == 'None'):
-                value = None
-            elif('.' in value):
-                value = float(value)
-            else:
-                value = int(value)
-
-            key = item[: item.index('=')]
-            key = key[key.rfind('.')+1:]
-
-            if(getattr(self, key) != value):
-                print('Setting {} to {}'.format(key, value), file=sys.stderr)
-            setattr(self, key, value)
-
-    def __init__(self, sys_args, log, root, *args):
+    def __init__(self, sys_args, *args):
         # Set the default options.
-        option_names = []
         for options in args:
             for option in options:
-                option_names.append(option['names'][0])
+                if(option['action'] == 'text' or option['action'] == 'blank'):
+                    continue
+
                 key = _to_key(option)
                 if(option['action'] == 'store_true'):
                     value = False
@@ -233,13 +218,12 @@ class ParseOptions():
                     value = True
                 elif(option['nargs'] != 1):
                     value = []
+                elif(option['default'] is None):
+                    value = None
                 else:
-                    value = option['default']
+                    value = option['type'](option['default'])
                 setattr(self, key, value)
 
-        dirpath = os.path.dirname(os.path.realpath(__file__))
-        self.set_config(os.path.join(dirpath, 'config.txt'), root)
-
         # Figure out command line options changed by user.
         my_list = []
         used_options = []
@@ -248,7 +232,6 @@ class ParseOptions():
         option_list = 'input'
         list_type = str
         i = 0
-        group = None
         while i < len(sys_args):
             item = sys_args[i]
             label = 'option' if item.startswith('--') else 'short'
@@ -261,8 +244,9 @@ class ParseOptions():
                     name_set = set()
                     for options in args:
                         for opt in options:
-                            for names in opt['names']:
-                                name_set.add(names)
+                            if(opt['action'] not in ['text', 'blank']):
+                                for names in opt['names']:
+                                    name_set.add(names)
                     return name_set
 
                 opt_list = all_names(args)
@@ -283,7 +267,7 @@ class ParseOptions():
                             item = self.parse_parameters(item, _op)
                     my_list.append(item)
                 else:
-                    log.error(error_message(args, item, label))
+                    raise ParserError(error_message(args, item, label))
             else:
                 # We found the option.
                 if(option_list is not None):
@@ -294,16 +278,13 @@ class ParseOptions():
                 my_list = []
 
                 if(option in used_options):
-                    log.error('Cannot repeat option {} twice.'.format(option['names'][0]))
+                    raise ParserError('Cannot repeat option {} twice.'.format(option['names'][0]))
 
                 used_options.append(option)
 
                 key = _to_key(option)
                 _set.append(key)
 
-                if(option['action'] == 'grouping'):
-                    group = key
-
                 nextItem = None if i == len(sys_args) - 1 else sys_args[i+1]
                 if(nextItem == '-h' or nextItem == '--help'):
                     print_option_help(args, option)
@@ -323,7 +304,8 @@ class ParseOptions():
                     if(option['choices'] is not None and value not in option['choices']):
                         option_name = option['names'][0]
                         my_choices = ', '.join(option['choices'])
-                        log.error('{} is not a choice for {}\nchoices are:\n  {}'.format(
+
+                        raise ParserError('{} is not a choice for {}\nchoices are:\n  {}'.format(
                             value, option_name, my_choices))
                     i += 1
                 setattr(self, key, value)
@@ -333,5 +315,5 @@ class ParseOptions():
             setattr(self, option_list, list(map(list_type, my_list)))
         setattr(self, '_set', _set)
         if(self.help):
-            print_program_help(root, args)
+            print_program_help(args)
             sys.exit()
